// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#include "IwGameActions.h"
#include "IwGameScene.h"
#include "IwGameResources.h"
#include "IwGameActor.h"
#include "IwGame.h"

//
//
//
//
// CIwGameActions implementation
//
//
//
//
void CIwGameActions::Execute(IIwGameXomlResource* source)
{
	if (Conditions != 0)
	{
		// Find the condition variable
		CIwGameXomlVariable* var = CIwGameXomlVariableManager::FindVariable(Conditions, source);
		if (var != NULL)
		{
			if (!var->isTrue())
				return;
		}
#if defined (_DEBUG)
		else
			CIwGameError::LogError("Error: condition variable not found - ", _Conditions.c_str());
#endif // _DEBUG
	}

	for (_Iterator it = begin(); it != end(); ++it)
	{
		CIwGameAction*	action = *it;
		unsigned int	method_name = action->MethodName;
		bool			condition = true;

		// If action has a condition then check that the condition is true before executing the action
		if ((*it)->Conditions != 0)
		{
			// Find the condition variable
			CIwGameXomlVariable* var = CIwGameXomlVariableManager::FindVariable((*it)->Conditions, source);

			if (var != NULL)
				condition = var->isTrue();
#if defined (_DEBUG)
			else
				CIwGameError::LogError("Error: condition variable not found - ", (*it)->_Conditions.c_str());
#endif // _DEBUG
		}
		if (condition)
		{
			// Search for action
			for (CIwGameXoml::_ActionsIterator it2 = IW_GAME_XOML->actions_begin(); it2 != IW_GAME_XOML->actions_end(); ++it2)
			{
				if ((*it)->MethodName == (*it2)->getActionNameHash())
				{
					// Execute action
					(*it2)->Execute(source, *it);
					break;
				}
			}
		}
	}
}

bool CIwGameActions::LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node)
{
	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();

		if (name_hash == CIwGameXomlNames::Name_Hash)
		{
			setName((*it)->GetValue().c_str());
		}
		else
		if (name_hash == CIwGameXomlNames::Condition_Hash)
		{
			Conditions = (*it)->GetValue().getHash();
#if defined(_DEBUG)
			_Conditions = (*it)->GetValue().c_str();
#endif	//_DEBUG
		}
	}

	// Load in nodes
	for (CIwGameXmlNode::_Iterator it2 = node->begin(); it2 != node->end(); ++it2)
	{
		unsigned int name_hash = (*it2)->GetName().getHash();
		if (name_hash == CIwGameXomlNames::Action_Hash)
		{
			CIwGameAction* action = new CIwGameAction();
			// Get action data
			for (CIwGameXmlNode::_AttribIterator it = (*it2)->attribs_begin(); it != (*it2)->attribs_end(); ++it)
			{
				unsigned int attrib_hash = (*it)->getName().getHash();

				if (attrib_hash == CIwGameXomlNames::Method_Hash)
				{
					(*it)->GetValue().ToLower();
					action->MethodName = (*it)->GetValue().getHash();
#if defined(_DEBUG)
					action->_MethodName = (*it)->GetValue();
#endif	//_DEBUG
				}
				else
				if (attrib_hash == CIwGameXomlNames::Param1_Hash)
				{
					action->Parameter1 = (*it)->GetValue();
				}
				else
				if (attrib_hash == CIwGameXomlNames::Param2_Hash)
				{
					action->Parameter2 = (*it)->GetValue();
				}
				else
				if (attrib_hash == CIwGameXomlNames::Param3_Hash)
				{
					action->Parameter3 = (*it)->GetValue();
				}
				else
				if (attrib_hash == CIwGameXomlNames::Condition_Hash)
				{
					action->Conditions = (*it)->GetValue().getHash();
#if defined(_DEBUG)
					action->_Conditions = (*it)->GetValue();
#endif	//_DEBUG
				}
			}
			// Save action
			Actions.push_back(action);
		}
	}

	// If we are declared inside a scene or actor then actions are local to the scene
	if (parent != NULL && (parent->getClassTypeHash() == CIwGameXomlNames::Scene_Hash || parent->getClassTypeHash() == CIwGameXomlNames::Actor_Hash))
	{
		if (parent->getClassTypeHash() == CIwGameXomlNames::Scene_Hash)
			((CIwGameScene*)parent)->getResourceManager()->addResource(this);
		else
			((CIwGameActor*)parent)->getScene()->getResourceManager()->addResource(this);
	}
	else
	{
		IW_GAME_GLOBAL_RESOURCES->getResourceManager()->addResource(this);
	}

	return true;
}




